/*
 * File         : BV_VideoApp.h
 * Author       : Tomasz Stachowiak
 * Created      : Fri Sep 25 17:56:53 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#ifndef __BV_VIDEOAPP_H
#define __BV_VIDEOAPP_H

#include <sys/types.h>
#include <sys/time.h>
#include <semaphore.h>

#include "rtp.h"
#include "BV.h"
#include "BV_Session_Control.h"
#include "BV_Application.h"
#include "BV_VideoCapThread.h"

#define MAX_MEMBERS 20

#define VIDEO_PLUGIN_PORT   34999

#define STOP    0
#define PLAY    1

#define INTRA_GAP 10

typedef enum {
   H263 = 0,
   H261,   
  videoTagInvalid,     /* Never used */
  videoTagPlay,    
  videoTagConfirm,    
  videoTagStop,
  videoTagExit,
  videoTagData,
  videoTagFrameDecoded,
  videoTagNewMember,
  YUV9
} VideoTag;

typedef struct _videoPluginFrame
{
  unsigned char num;
  struct timeval time;
  char data[1];
} VideoPluginFrame;

class BV_VideoApp : public BV_Application
{
 public:
  BV_VideoApp();
  ~BV_VideoApp();
  int process_message(RTPheader *header, char *body, int len);
  int send_video();
  void play();
  void stop();
  void update_options(Options *options);

 protected:

  int displayH263(BV_Member *mbr, char *data, int len);
  int displayH261(BV_Member *mbr, char *data, int len);
  int displayYUV9(BV_Member *mbr, char *data, int len);
  int sendH263();
  int sendH261();
  int send_video_data(int tag, char *data, int len);
  void wait_for_next_frame(BV_Member *mbr);

  int state_;
  int compression_;
  BV_VideoCapThread *cap_thread_;
  int first;
  int frame_no;
  int msg_no;

  BV_Session_Control *session_;
  void *main_form_;

  sem_t semH263_;
  sem_t semH261_;
  sem_t sem_;

  double wait_time_;
  double kbps_limit_;
  int fps_max_;
};

#endif

